

import site.ycsb.DB;
import site.ycsb.DBException;
import site.ycsb.ByteArrayByteIterator;
import site.ycsb.ByteIterator;
import site.ycsb.Status;
import site.ycsb.StringByteIterator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class HpdosClient extends DB {

    private static final Logger LOGGER = LoggerFactory.getLogger(HpdosClient.class);

    public HpdosClient() {
    }

    JClient jclient;

    public void init() throws DBException {

        System.out.println(System.getProperty("user.dir"));
        Properties prop = getProperties();
        prop.forEach((k, v) -> System.out.println(k + "" + v));
        System.out.println("init hpdos");
        jclient = JClient.getInstance();
        System.out.println("jclient"+jclient);
        //synchronized (HpdosClient.class) {
        //    if (jclient == null) {
        //        System.loadLibrary("hpdosclient");
        //        jclient = new JClient();
        //        jclient.endpointGroup = jclient.createEndpointGroup("../prop.config");
        //    }
        //}
        LOGGER.info("endpoint" + jclient.endpointGroup);
    }

    public void cleanup() throws DBException {
        jclient.closeEndpointGroup(jclient.endpointGroup);
        LOGGER.info("closing");
    }

    // Read a single record
    public Status read(String table, String key, Set<String> fields, Map<String, ByteIterator> result) {
        System.out.println("Read" + key.length());
        byte res[] = jclient.get(jclient.endpointGroup, key.getBytes());
        if (res == null)
            return Status.NOT_FOUND;
        if(fields == null)
        {
            return Status.BAD_REQUEST;
        }
        Iterator<String> it = fields.iterator();
        if (it.hasNext())
            result.put(fields.iterator().next(), new ByteArrayByteIterator(res, 0, res.length));
        return Status.OK;
    }

    // Perform a range scan
    public Status scan(String table, String startkey, int recordCount, Set<String> fieds,
            Vector<HashMap<String, ByteIterator>> result) {
        return Status.NOT_IMPLEMENTED;
    }

    // Update a single record
    public Status update(String table, String key, Map<String, ByteIterator> values) {
        System.out.println("update" + key.getBytes().length );
        try {
            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            for (ByteIterator v : values.values()) {
                try {
                    stream.write(v.toArray());
                } catch (IOException ex) {
                    LOGGER.error(ex.getMessage());
                    return Status.ERROR;
                }
            }
            int val = jclient.put(jclient.endpointGroup, key.getBytes(), stream.toByteArray());
            if (val == -1)
            {
                System.out.println("got -1\n");
                return Status.BAD_REQUEST;
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            return Status.ERROR;
        }
        return Status.OK;
    }

    // Insert a single record
    public Status insert(String table, String key, Map<String, ByteIterator> values) {
        return Status.OK;
    }

    // Delete a single record
    public Status delete(String table, String key) {
        int status = jclient.delete(jclient.endpointGroup, key.getBytes());
        if (status == -1    )
            return Status.ERROR;
        return Status.OK;
    }

}

// -Dcheckstyle.skip