#ifndef __RDMACLIENTENDPOINT__
#define __RDMACLIENTENDPOINT__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <netdb.h>
#include <iostream>
#include <errno.h>
#include <arpa/inet.h>
//#include <boost/lockfree/queue.hpp>
#include <queue>
#include <map>
#include <mutex>
#include "Buffer.hpp"
#include "Logger.hpp"
#include "RdmaEndpointGroup.hpp"
#include "MessageFormats.hpp"
#include "RdmaFuture.hpp"

class RdmaClientEndpoint
{

    static int CONN_STATE_INITIALIZED;
    static int CONN_STATE_ADDR_RESOLVED;
    static int CONN_STATE_ROUTE_RESOLVED;
    static int CONN_STATE_RESOURCES_ALLOCATED;
    static int CONN_STATE_CONNECTED;
    static int CONN_STATE_PARTIAL_CLOSED;
    static int CONN_STATE_CLOSED;

    struct rdma_cm_id *_cm_id = NULL;
    struct ibv_pd *_protectionDomain;

    RdmaEndpointGroup *_group;
    int _sendQueueSize;
    int _recvQueueSize;
    int _sendMsgSize;
    int _recvMsgSize;
    int _state;
    int _maxInLine;
    int _timeoutMs;

    const char *_connData;

    char *_sendBuff = NULL;
    char *_recvBuff = NULL;
    struct ibv_mr *_sendMr = NULL;
    struct ibv_mr *_recvMr = NULL;

    // boost::lockfree::queue<char *> *_sendBuffers;
    std::queue<char *> _sendBuffers;
    std::mutex _sendBuffersM;

    std::map<uint64_t, RdmaFuture *> futures;
    std::mutex _futureMutex;

    void connect();
    void registerMemory();
    void createQueuePairs();

public:
    std::atomic<uint64_t> _requestId{12};

    RdmaClientEndpoint(struct rdma_cm_id *id, RdmaEndpointGroup *group, int sendQueueSize, int recvQueueSize,
                       int sendMsgSize, int recvMsgSize, int maxInLine, int timeout);
    void processCmEvent(struct rdma_cm_event *event);
    void connect(const char *ip, const char *port, const char *connData);
    bool isConnected();
    void close();
    void completeClose();

    void processSendComp(struct ibv_wc &);
    void processRecvComp(struct ibv_wc &);

    int sendMessage(const char *buffer, int size);
    RdmaFuture *put(const char *key, int keySize, const char *value, int valueSize);
    RdmaFuture *get(const char *key, int keySize);
    RdmaFuture *deleteKey(const char *key, int keySize);
};
#endif