#ifndef __RDMACLIENTENDPOINTGROUP__
#define __RDMACLIENTENDPOINTGROUP__

#include <vector>
#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <errno.h>
#include <unordered_map>
#include <chrono>
#include <thread>
#include <mutex>
#include "RdmaEndpointGroup.hpp"
#include "RdmaCqProcessor.hpp"
#include "RdmaCmProcessor.hpp"
#include "RdmaClientEndpoint.hpp"
#include "Properties.hpp"
class RdmaClientEndpointGroup : public RdmaEndpointGroup
{
    RdmaCmProcessor *_cmProcessor = NULL;
    RdmaCqProcessor *_cqProcessor = NULL;
    // struct rdma_cm_id *_cm_id;

    int _sendQueueSize;
    int _recvQueueSize;
    int _compQueueSize;
    int _sendMsgSize;
    int _recvMsgSize;
    int _maxInLine;
    int _timeoutMs;

    std::vector<RdmaClientEndpoint*> _clientEps;
    int _clients;
    int _counter;
    std::mutex _counterMutex;

public:
    RdmaClientEndpointGroup(int sendQueueSize, int recvQueueSize, int compQueueSize, int sendMsgSize,
                            int recvMsgSize, int maxInLine, int timeout);
    void processCmEvent(struct rdma_cm_event *event);
    struct ibv_cq *createCq(struct rdma_cm_id *id);
    RdmaClientEndpoint *createEndpoint();
    void createClientEps(Properties *);
    /* functions return RDMAFuture whose get will wait until data is put
     * to avoid busy waiting cv is used*/
    RdmaFuture* put(const char *key, int keySize, const char *value, int valueSize);
    RdmaFuture* get(const char *key, int keySize);
    RdmaFuture* deleteKey(const char *key, int keySize);
    void close();  
    
};
#endif