#include <stdio.h>
#include "JClient.h" // generated by javah
#include "Logger.hpp"
#include "RdmaClientEndpoint.hpp"
#include "RdmaClientEndpointGroup.hpp"
#include <unistd.h>

JNIEXPORT jlong JNICALL Java_JClient_createEndpointGroup(JNIEnv *jenv, jobject jobj, jstring jpath)
{
	//	exec_gdb();
	jboolean iscopy;
	const char *path = jenv->GetStringUTFChars(jpath, &iscopy);
	// std::cout<<path<<"\n";

	Properties *prop = new Properties(path);
	int sendQS = stoi(prop->getValue("sendQS"));
	int recvQS = stoi(prop->getValue("recvQS"));
	int compQS = stoi(prop->getValue("compQS"));
	int sendMS = stoi(prop->getValue("sendMS"));
	int recvMS = stoi(prop->getValue("recvMS"));

	std::cout << "Creating EndpointGroup SendQS " << sendQS << " recvQS " << recvQS << " compQS ";
	std::cout << compQS << " sendMS " << sendMS << " recvMS " << recvMS << "\n";
	RdmaClientEndpointGroup *group = new RdmaClientEndpointGroup(sendQS, recvQS, compQS, sendMS, recvMS, 0, 1000);
	std::cout << "Connecting Endpoints\n";
	group->createClientEps(prop);
	delete prop;

	return reinterpret_cast<jlong>(group);
}

JNIEXPORT void JNICALL Java_JClient_closeEndpointGroup(JNIEnv *jenv, jobject jobj, jlong jclient)
{
	RdmaClientEndpointGroup *group = reinterpret_cast<RdmaClientEndpointGroup *>(jclient);
	group->close();
	//delete group;
	// group->close();
}

JNIEXPORT jbyteArray JNICALL Java_JClient_get(JNIEnv *jenv, jobject jobj, jlong jclient,
											  jbyteArray jkey)
{
	/* Get Key form Java Side*/
	int keyLen = jenv->GetArrayLength(jkey);
	char *key = new char[keyLen];
	jenv->GetByteArrayRegion(jkey, 0, keyLen, reinterpret_cast<jbyte *>(key));
	if (jenv->ExceptionCheck())
	{
		CPPLog::LOG_ERROR("exception occurs in jni get");
		delete[] key;
	}

	/* Get data from server*/
	RdmaClientEndpointGroup *group = reinterpret_cast<RdmaClientEndpointGroup *>(jclient);
	auto future = group->get(key, keyLen);
	delete[] key;
	/* Handle failed get request*/
	if (future == nullptr)
	{
		jbyteArray jvalue = jenv->NewByteArray(0);
		return jvalue;
	}
	/* Create Java Byte Array and send data to java side
	 * after copying data from c char array to java byte array
	 */
	//std::cout << "get future.get()\n";
	char *data = future->get();
	delete future;
	//std::cout << "g future.get()\n";
	struct MessageHeader *response = (struct MessageHeader *)data;
	if (response->type == MessageType::FAILURE)
	{
		jbyteArray jvalue = jenv->NewByteArray(0);
		delete[] data;
		return jvalue;
	}
	jbyteArray jvalue = jenv->NewByteArray(response->valueSize);
	// std::cout << "value size " << response->valueSize << " " << (data + MessageHeaderSize);
	jenv->SetByteArrayRegion(jvalue, 0, response->valueSize - 1,
							 reinterpret_cast<const jbyte *>(data + MessageHeaderSize));
	delete[] data;
	return jvalue;
}

JNIEXPORT jint JNICALL Java_JClient_put(JNIEnv *jenv, jobject jobj, jlong jclient,
										jbyteArray jkey, jbyteArray jval)
{
	/*getting key from java side */
	int keyLen = jenv->GetArrayLength(jkey);
	char *key = new char[keyLen];
	jenv->GetByteArrayRegion(jkey, 0, keyLen, reinterpret_cast<jbyte *>(key));
	if (jenv->ExceptionCheck())
	{
		CPPLog::LOG_ERROR("exception occurs in jni put");
		delete[] key;
		return -1;
	}

	/*getting value from java side */
	int valLen = jenv->GetArrayLength(jval);
	char *val = new char[valLen];
	jenv->GetByteArrayRegion(jval, 0, valLen, reinterpret_cast<jbyte *>(val));
	// std::cout << "put got " << "key "<<key<<" val " <<val << " len " << valLen;
	if (jenv->ExceptionCheck())
	{
		CPPLog::LOG_ERROR("exception occurs in jni put");
		delete[] key;
		delete[] val;
		return -1;
	}

	/* Putting data to server*/
	RdmaClientEndpointGroup *group = reinterpret_cast<RdmaClientEndpointGroup *>(jclient);
	auto future = group->put(key, keyLen, val, valLen);
	delete[] key;
	delete[] val;

	if (future == nullptr)
	{
		return -1;
	}
	//std::cout << "put future.get()\n";
	auto data = future->get();
	delete future;
	//std::cout << "p future.get()\n";
	struct MessageHeader *response = (struct MessageHeader *)data;
	//std::cout<<"server response "<<response->type<<"\n";
	if (response->type == MessageType::SUCCESS)
	{
		// Currently 0 indicate succces and remaing are failure more status code can be added in future
		return 0;
	}
	return -1;
}

JNIEXPORT jint JNICALL Java_JClient_delete(JNIEnv *jenv, jobject jobj, jlong jclient,
										   jbyteArray jkey)
{
	/*getting key from java side */
	int keyLen = jenv->GetArrayLength(jkey);
	char *key = new char[keyLen];
	jenv->GetByteArrayRegion(jkey, 0, keyLen, reinterpret_cast<jbyte *>(key));
	if (jenv->ExceptionCheck())
	{
		CPPLog::LOG_ERROR("exception occurs in jni delete");
		delete[] key;
	}

	/* deleting data from server*/
	RdmaClientEndpointGroup *group = reinterpret_cast<RdmaClientEndpointGroup *>(jclient);
	auto future = group->deleteKey(key, keyLen);
	delete[] key;

	if (future == nullptr)
	{
		return -1;
	}
	//std::cout << "delete future.get()\n";
	auto data = future->get();
	delete future;
	//std::cout << "d future.get()\n";
	struct MessageHeader *response = (struct MessageHeader *)data;
	if (response->type == MessageType::FAILURE)
	{
		return -1;
	}
	return 0;
}

int gdb_process_pid = 0;

void exec_gdb()
{
	// Create child process for running GDB debugger
	int pid = fork();

	if (pid < 0) /* error */
	{
		abort();
	}
	else if (pid) /* parent */
	{
		// Application process

		gdb_process_pid = pid; // save debugger pid
		sleep(10);			   /* Give GDB time to attach */

		// Continue the application execution controlled by GDB
	}
	else /* child */
	{
		// GDB process. We run DDD GUI wrapper around GDB debugger

		std::stringstream args;
		// Pass parent process id to the debugger
		args << "--pid=" << getppid();

		// Invoke DDD debugger
		execl("ddd", "ddd", "--debugger", "gdb", args.str().c_str(), (char *)0);

		// Get here only in case of DDD invocation failure
		std::cerr << "\nFailed to exec GDB (DDD)\n"
				  << std::endl;
	}
}

/*
bool rocksdb_put_helper(JNIEnv* env, ROCKSDB_NAMESPACE::DB* db,
						const ROCKSDB_NAMESPACE::WriteOptions& write_options,
						ROCKSDB_NAMESPACE::ColumnFamilyHandle* cf_handle,
						jbyteArray jkey, jint jkey_off, jint jkey_len,
						jbyteArray jval, jint jval_off, jint jval_len) {
  jbyte* key = new jbyte[jkey_len];
  env->GetByteArrayRegion(jkey, jkey_off, jkey_len, key);
  if (env->ExceptionCheck()) {
	// exception thrown: ArrayIndexOutOfBoundsException
	delete[] key;
	return false;
  }

  jbyte* value = new jbyte[jval_len];
  env->GetByteArrayRegion(jval, jval_off, jval_len, value);
  if (env->ExceptionCheck()) {
	// exception thrown: ArrayIndexOutOfBoundsException
	delete[] value;
	delete[] key;
	return false;
  }

*/
