#include "RdmaClientEndpointGroup.hpp"

RdmaClientEndpointGroup::RdmaClientEndpointGroup(int sendQueueSize, int recvQueueSize, int compQueueSize, int sendMsgSize,
                                                 int recvMsgSize, int maxInLine, int timeout)
    : _sendQueueSize(sendQueueSize), _recvQueueSize(recvQueueSize), _compQueueSize(compQueueSize),
      _sendMsgSize(sendMsgSize), _recvMsgSize(recvMsgSize), _maxInLine(maxInLine), _timeoutMs(timeout)
{
    _cmProcessor = new RdmaCmProcessor(this);
    _cmProcessor->start();
    _counter = 0;
}

void RdmaClientEndpointGroup::processCmEvent(struct rdma_cm_event *event)
{
    std::cout << "RdmaClientEndpointGroup :  event" << rdma_event_str(event->event) << std::endl;
    if (event->id != NULL && event->id->context != NULL)
    {
        ((RdmaClientEndpoint *)event->id->context)->processCmEvent(event);
        if (event->event == RDMA_CM_EVENT_ADDR_RESOLVED)
        {
            _cqProcessor->registerEp(event->id->qp->qp_num, ((RdmaClientEndpoint *)event->id->context));
        }
        if (event->event == RDMA_CM_EVENT_DISCONNECTED)
        {
            _cqProcessor->deRegisterEp(event->id->qp->qp_num);
        }
    }
    else
    {
        std::ostringstream ss;
        ss<<"RdmaClientEndpointGroup : Not able to procces CM EVent" << rdma_event_str(event->event) ;
        ss << " id "<<event->id << " listen id"<< event->listen_id;
        CPPLog::LOG_ERROR(ss);
    }
}

RdmaClientEndpoint *RdmaClientEndpointGroup::createEndpoint()
{
    struct rdma_cm_id *id = _cmProcessor->createId();
    RdmaClientEndpoint *endpoint = new RdmaClientEndpoint(id, this,
                                                          _sendQueueSize, _recvQueueSize,
                                                          _sendMsgSize, _recvMsgSize, _maxInLine, _timeoutMs);
    /* Setting Endpoint in cm_id context so that whenever cm event come we can get endpoint
     */
    id->context = (void *)endpoint;
    return endpoint;
}

struct ibv_cq *RdmaClientEndpointGroup::createCq(struct rdma_cm_id *id)
{
    if (_cqProcessor == NULL)
    {
        _cqProcessor = new RdmaCqProcessor(id->verbs, _compQueueSize);
        _cqProcessor->start();
    }
    return _cqProcessor->getCq();
}

void RdmaClientEndpointGroup::createClientEps(Properties *prop)
{
    _clients = stoi(prop->getValue("NSERVERS"));
    std::cout << "clients" << _clients << "\n";
    for (int i = 0; i < _clients; i++)
    {
        std::cout << "creating client for " << prop->getValue("SERVER_IP" + std::to_string(i + 1));
        std::cout << ":" << prop->getValue("SERVER_PORT" + std::to_string(i + 1)) << " \n";
        RdmaClientEndpoint *ep = createEndpoint();
        ep->connect((prop->getValue("SERVER_IP" + std::to_string(i + 1))).c_str(),
                    (prop->getValue("SERVER_PORT" + std::to_string(i + 1))).c_str(), "sal");
        _clientEps.push_back(ep);
    }
    for (int i = 0; i < _clients; i++)
    {
        while (!_clientEps[i]->isConnected())
        {
            continue;
        }
    }
    std::cout << "connected\n";
}

RdmaFuture *RdmaClientEndpointGroup::put(const char *key, int keySize, const char *value, int valueSize)
{
    // std::cout<<"Client 1\n";
    int id = _counter;
    _counter = (_counter + 1) % _clients;
    // std::cout<<"Client 2"<<_counter<<" "<<_clientEps.size()<<"\n";
    auto future = _clientEps[id]->put(key, keySize, value, valueSize);
    // std::cout<<"endpoint get\n";
    return future;
}
RdmaFuture *RdmaClientEndpointGroup::get(const char *key, int keySize)
{
    int id = _counter;
    _counter = (_counter + 1) % _clients;
    return _clientEps[id]->get(key, keySize);
}
RdmaFuture *RdmaClientEndpointGroup::deleteKey(const char *key, int keySize)
{
    int id = _counter;
    _counter = (_counter + 1) % _clients;
    return _clientEps[id]->deleteKey(key, keySize);
}

void RdmaClientEndpointGroup::close()
{
    if (_cmProcessor != nullptr)
    {
        _cmProcessor->close();
        delete _cmProcessor;
    }
    if (_cqProcessor != nullptr)
    {
        _cqProcessor->close();
        delete _cqProcessor;
    }
    for (auto ep : _clientEps)
    {
        ep->close();
        ep->completeClose();
        delete ep;
    }
}